#include "defines.h"

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

extern external_objects extobj[NUMOBJS];
extern short debug;

void read_raw_obj(char *filename, int id) {
	FILE *fp;
	int i=0;
	char templine[MAXLINE_READ];
	char dummybuffer[128];
	int tempcolorid;
	char subobj[64];

	// Try to open and Check for error
	if((fp = fopen((char *)filename, "r")) == NULL) {
		printf("Debug: Opening \"%s\" ... \n", filename);
		perror("Error opening model");
		exit(1);
	} else {
		if(debug)
			printf("Success: Now reading model from \"%s\" ...\n", filename);
	}

	// Find out how many vertices we have
	while(fgets(templine, MAXLINE_READ, fp)) {
		if(templine[0] == 45 || (templine[0] < 58 && templine[0] > 47))
			i++;
	}
	// allocate memory
	i *= 3;
	if(debug)
		printf("Debug: Now allocating memory for %d vertices ... \n", i);
	extobj[id].vx = malloc(sizeof(GLfloat)*i);
	extobj[id].vy = malloc(sizeof(GLfloat)*i);
	extobj[id].vz = malloc(sizeof(GLfloat)*i);
	extobj[id].colorid = malloc(sizeof(int)*i);
	
	// Reads the file per-line until fgets returns NULL
	rewind(fp);
	i=0;
	while(fgets(templine, MAXLINE_READ, fp))
	{
		// I don't like endlines, trash em
		templine[strlen(templine)-2] = 0;

		// Object with a defined material
		if(templine[0] == '_')
			sscanf(templine, "_%d_%s", &tempcolorid, dummybuffer);
		// A vertex ...
		else if(templine[0] == 45 || (templine[0] < 58 && templine[0] > 47))
		{
			// We have 3 vertices

			if(sscanf(templine, "%f %f %f   %f %f %f   %f %f %f",
						&extobj[id].vx[i],
						&extobj[id].vy[i],
						&extobj[id].vz[i],
						&extobj[id].vx[i+1],
						&extobj[id].vy[i+1],
						&extobj[id].vz[i+1],
						&extobj[id].vx[i+2],
						&extobj[id].vy[i+2],
						&extobj[id].vz[i+2]) < 0)
			{
				printf("Ack, error reading \"%s\" from \"%s\"\n",
						subobj, filename);
				exit(1);
			}
			else
			{
				// Assign the current color to all vertices
				extobj[id].colorid[i]=
					extobj[id].colorid[i+1]=
					extobj[id].colorid[i+2]=tempcolorid;
					
				i += 3;
			}
		}
		// If we have an object without a defined material
		else
		{
			// Default is gengrey
			tempcolorid = 14;
			strcpy(subobj, templine);
			if(debug)
			{
				printf("Debug: Now processing subobject \"%s\" from \"%s\"\n",
						subobj, filename);
			}
		}
	}
	if(debug)
		printf("Total verticies read = %d\n", i);
	extobj[id].numvertices = i;

	fclose(fp);
}
