// Draws a Lambda Shuttle

// Main header file
#include "defines.h"

// Some file-specific prototypes
void base_shuttle(float);
void top_fin_shuttle(float);
void shuttlefins(float, float);
void side_fin(float);
void make_cockpit(float);

extern GLUquadricObj* gluObj;

void legs(float, float, float, float);

// Draws the legs
void legs(float g, float a, float b, float ang)
{
	// Wings Down
	if(ang > 135)
		ang = 135;
	
	// Wings Up
	if(ang < 0)
		ang = 0;

	// Make the legs
	glPushMatrix();
		glTranslatef(a, g/12, b);
		glRotatef(90, 1.0, 0.0, 0.0);
		glRotatef(ang*(4.0/3.0), 1.0, 0.0, 0.0);
		gluCylinder(gluObj, g/16, g/16, g/4-g/96, 6, 6);
	glPopMatrix();
}

// Creates a Shuttlecraft
void shuttle(float g, float ang)
{
	// Base
	base_shuttle(g);
	top_fin_shuttle(g);
	shuttlefins(g, ang);

	// Legs
	legs(g, g/4, -g/4, ang);
	legs(g, g/4, g/4, ang);
	legs(g, -g/4, g/4, ang);
	legs(g, -g/4, -g/4, ang);

	// Make the cockpit
	glPushMatrix();
		glTranslatef(0.0, 0.0, g/2);
		tdrect(-g/5.6, g/11, 0.0, g/5.6, g/4, g/3);
		glPushMatrix();
			glTranslatef(0.0, 0.0, g/5);
			glRotatef(5.0, 1.0, 0.0, 0.0);			
			make_cockpit(g);
		glPopMatrix();
	glPopMatrix();
}

void make_cockpit(float g)
{
	glPushMatrix();
		glTranslatef(0.0, g/14, 0.0);
		// back plate
		glBegin(GL_QUADS);
			glNormal3f(0.0, 0.0, -1.0);
			glVertex3f(-g/4, 0.0, 0.0);
			glVertex3f(-g/4, g/3.8, 0.0);
			glVertex3f(g/4, g/3.8, 0.0);
			glVertex3f(g/4, 0.0, 0.0);
		glEnd();

		// Right triangle
		glBegin(GL_QUADS);
			glNormal3f(1.0, 0.0, 0.0);
			glVertex3f(g/4, 0.0, 0.0);
			glVertex3f(g/4, g/3.8, 0.0);
			glVertex3f(g/4, g/10, g/1.8);
			glVertex3f(g/4, 0.0, g/1.8);
		glEnd();

		// Left triangle
		glBegin(GL_QUADS);
			glNormal3f(-1.0, 0.0, 0.0);
			glVertex3f(-g/4, 0.0, 0.0);
			glVertex3f(-g/4, g/3.8, 0.0);
			glVertex3f(-g/4, g/10, g/1.8);
			glVertex3f(-g/4, 0.0, g/1.8);
		glEnd();

		// Bottom
		glBegin(GL_QUADS);
			glNormal3f(0.0, -1.0, 0.0);
			glVertex3f(-g/4, 0.0, 0.0);
			glVertex3f(-g/4, 0.0, g/1.8);
			glVertex3f(g/4, 0.0, g/1.8);
			glVertex3f(g/4, 0.0, 0.0);
		glEnd();

		// Top
		glPushMatrix();
			glTranslatef(0.0, g/3.8, 0.0);
			glRotatef(16.36666, 1.0, 0.0, 0.0);
			glBegin(GL_QUADS);
				glNormal3f(0.0, 1.0, 0.0);
				glVertex3f(-g/4, 0.0, 0.0);
				glVertex3f(-g/4, 0.0, sqrt((g/1.8)*(g/1.8) +
							(g/6.12903)*(g/6.12903)));
				glVertex3f(g/4, 0.0,sqrt((g/1.8)*(g/1.8) +
							(g/6.12903)*(g/6.12903)));
				glVertex3f(g/4, 0.0, 0.0);
			glEnd();
			setMaterials(&Obsidian, GL_FRONT_AND_BACK);
			tdrect(-g/4.01, 0.0, g/3, g/4.01, g/64, g/14);
			setMaterials(&gengrey, GL_FRONT_AND_BACK);
		glPopMatrix();

		// Front grill or something
		glBegin(GL_QUADS);
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3f(g/4, 0.0, g/1.8);
			glVertex3f(g/4, g/10, g/1.8);
			glVertex3f(-g/4, g/10, g/1.8);
			glVertex3f(-g/4, 0.0, g/1.8);
		glEnd();
	glPopMatrix();
}

void shuttlefins(float g, float ang)
{
	// Left Fin
	glPushMatrix();
		glTranslatef(-g/2, g/12, 0.0);
		glPushMatrix();
			glRotatef(ang, 0.0, 0.0, 1.0);
			side_fin(g);
		glPopMatrix();
	glPopMatrix();

	// Right Fin
	glPushMatrix();
		glTranslatef(g/2, g/12, 0.0);
		glPushMatrix();
			glRotatef(-ang, 0.0, 0.0, 1.0);
			side_fin(g);
		glPopMatrix();
	glPopMatrix();
}

void side_fin(float g)
{
	GLfloat temporary;

	// Make the right plate first
	glPushMatrix();
		glTranslatef(g/24, 0.0, 0.0);
		glBegin(GL_POLYGON);
			glNormal3f(1.0, 0.0, 0.0);
			glVertex3f(0.0, 0.0, -g/2.1);
			glVertex3f(0.0, g/0.8, -g/2.1);
			glVertex3f(0.0, g/0.8, 0.0);
			glVertex3f(0.0, 0.0, g/2.1);
		glEnd();
	glPopMatrix();

	// Make the left plate
	glPushMatrix();
		glTranslatef(-g/24, 0.0, 0.0);
		glBegin(GL_POLYGON);
			glNormal3f(-1.0, 0.0, 0.0);
			glVertex3f(0.0, 0.0, -g/2.1);
			glVertex3f(0.0, g/0.8, -g/2.1);
			glVertex3f(0.0, g/0.8, 0.0);
			glVertex3f(0.0, 0.0, g/2.1);
		glEnd();
	glPopMatrix();

	// Make the junk that connects the two plates
	glPushMatrix();
		glTranslatef(0, 0, g/2.1);
		glRotatef(-20.8545, 1.0, 0.0, 0.0);
		glBegin(GL_POLYGON);
			temporary = sqrt((g/0.8)*(g/0.8)+((g/2.1)*(g/2.1)));
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3f(g/24, 0.0, 0);
			glVertex3f(g/24, temporary, 0);
			glVertex3f(-g/24, temporary, 0);
			glVertex3f(-g/24, 0, 0);
		glEnd();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0, g/0.8, 0.0);
		glBegin(GL_POLYGON);
			glNormal3f(0.0, 1.0, 0.0);
			glVertex3f(g/24, 0.0, 0.0);
			glVertex3f(g/24, 0.0, -g/2.1);
			glVertex3f(-g/24, 0.0, -g/2.1);
			glVertex3f(-g/24, 0.0, 0.0);
		glEnd();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0, 0, -g/2.1);
		glBegin(GL_POLYGON);
			glNormal3f(0.0, 0.0, -1.0);
			glVertex3f(g/24, 0.0, 0.0);
			glVertex3f(g/24, g/0.8, 0.0);
			glVertex3f(-g/24, g/0.8, 0.0);
			glVertex3f(-g/24, 0.0, 0.0);
		glEnd();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.0, 0.0, -g/2.1);
		glBegin(GL_POLYGON);
			glNormal3f(0.0, -1.0, 0.0);
			glVertex3f(g/24, 0.0, (2*g)/2.1);
			glVertex3f(g/24, 0.0, 0.0);
			glVertex3f(-g/24, 0.0, 0.0);
			glVertex3f(-g/24, 0.0, (2*g)/2.1);
		glEnd();
	glPopMatrix();
}

void top_fin_shuttle(float g)
{
	GLfloat temporary;

	// Make the right plate first
	glPushMatrix();
		glTranslatef(g/24, g/2, 0.0);

		glBegin(GL_POLYGON);
			glNormal3f(1.0, 0.0, 0.0);

			glVertex3f(0.0, 0.0, -g/2);
			glVertex3f(0.0, g/0.8, -g/2);
			glVertex3f(0.0, g/0.8, 0.0);
			glVertex3f(0.0, 0.0, g/1.2);
		glEnd();
	glPopMatrix();

	// Make the left plate
	glPushMatrix();
		glTranslatef(-g/24, g/2, 0.0);

		glBegin(GL_POLYGON);
			glNormal3f(-1.0, 0.0, 0.0);

			glVertex3f(0.0, 0.0, -g/2);
			glVertex3f(0.0, g/0.8, -g/2);
			glVertex3f(0.0, g/0.8, 0.0);
			glVertex3f(0.0, 0.0, g/1.2);
		glEnd();
	glPopMatrix();

	// Make the junk that connects the two plates
	glPushMatrix();
		glTranslatef(0, g/2, g/1.2);
		glRotatef(-33.6901, 1.0, 0.0, 0.0);
		glBegin(GL_POLYGON);
			temporary = sqrt((g/0.8)*(g/0.8)+((g/1.2)*(g/1.2)));
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3f(g/24, 0.0, 0);
			glVertex3f(g/24, temporary, 0);
			glVertex3f(-g/24, temporary, 0);
			glVertex3f(-g/24, 0, 0);
		glEnd();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0, (g/2)+(g/0.8), 0.0);
		glBegin(GL_POLYGON);
			glNormal3f(0.0, 1.0, 0.0);
			glVertex3f(g/24, 0.0, 0.0);
			glVertex3f(g/24, 0.0, -g/2);
			glVertex3f(-g/24, 0.0, -g/2);
			glVertex3f(-g/24, 0.0, 0.0);
		glEnd();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0, g/2, -g/2);
		glBegin(GL_POLYGON);
			glNormal3f(0.0, 0.0, -1.0);
			glVertex3f(g/24, 0.0, 0.0);
			glVertex3f(g/24, g/0.8, 0.0);
			glVertex3f(-g/24, g/0.8, 0.0);
			glVertex3f(-g/24, 0.0, 0.0);
		glEnd();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0, g/2, -g/2);
		glBegin(GL_POLYGON);
			glNormal3f(0.0, -1.0, 0.0);
			glVertex3f(g/24, 0.0, g/2+g/1.2);
			glVertex3f(g/24, 0.0, 0.0);
			glVertex3f(-g/24, 0.0, 0.0);
			glVertex3f(-g/24, 0.0, g/2+g/1.2);
		glEnd();
	glPopMatrix();
}

void base_shuttle(float g)
{
	// Make the base body
	// Bottom Face
	tdrect(-g/2, 0, -g/2, g/2, g/6, g/2);

	// Now make the slice in the middle
	tdrect(-(3.2/10.0)*g, g/6, -g/2, (3.2/10.0)*g, g/3, g/2);

	// Now make the third slice on the top
	tdrect(-g/10, g/4, -g/2, g/10, g/2, g/4);

	// Now make the slanty thing on the front of the base
	// top triang
	glPushMatrix();
		glBegin(GL_POLYGON);
			glNormal3f(0.0, 1.0, 0.0);
			glVertex3f(g/10, g/2, g/4);
			glVertex3f(-g/10, g/2, g/4);
			glVertex3f(-g/10, g/3, g/2);
			glVertex3f(g/10, g/3, g/2);
		glEnd();
		glRotatef(atan((1.0/6.0)/(0.25)), 1.0, 0.0, 0.0);
	glPopMatrix();

	// Side triangles
	glBegin(GL_TRIANGLES);
		glNormal3f(-1.0, 0.0, 0.0);
		glVertex3f(-g/10, g/3, g/2);
		glVertex3f(-g/10, g/2, g/4);
		glVertex3f(-g/10, g/3, g/4);
	glEnd();
	glBegin(GL_TRIANGLES);
		glNormal3f(1.0, 0.0, 0.0);
		glVertex3f(g/10, g/2, g/4);
		glVertex3f(g/10, g/3, g/4);
		glVertex3f(g/10, g/3, g/2);
	glEnd();

	// Now make the rounded sides for the base slice
	// Cylinders!
	// Right side
	glPushMatrix();
		glTranslatef(-g/2, g/12, -g/2);
		gluCylinder(gluObj, g/12, g/12, g, 16, 16);
	glPopMatrix();

	glPushMatrix();
		glTranslatef(-g/2, g/12, g/2);
		gluSphere(gluObj, g/12, 6, 6);
	glPopMatrix();

	// Left side
	glPushMatrix();
		glTranslatef(g/2, g/12, -g/2);
		gluSphere(gluObj, g/12, 6, 6);
		gluCylinder(gluObj, g/12, g/12, g, 16, 16);
	glPopMatrix();

	// Back side
	glPushMatrix();
		glRotatef(90, 0.0, 1.0, 0.0);
		glTranslatef(g/2, g/12, -g/2);
		gluSphere(gluObj, g/12, 6, 6);
		gluCylinder(gluObj, g/12, g/12, g, 16, 16);
		glPushMatrix();
			setMaterials(&bluengines, GL_FRONT_AND_BACK);
			glTranslatef(g/48, 0.0, g/12);
			gluCylinder(gluObj, g/12, g/12, g/3, 16, 16);
			setMaterials(&gengrey, GL_FRONT_AND_BACK);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(g/48, 0.0, g-g/12);
			glPushMatrix();
				setMaterials(&bluengines, GL_FRONT_AND_BACK);
				glRotatef(180, 0.0, 1.0, 0.0);
				gluCylinder(gluObj, g/12, g/12, g/3, 16, 16);
				setMaterials(&gengrey, GL_FRONT_AND_BACK);
			glPopMatrix();
		glPopMatrix();
	glPopMatrix();

	// Front side
	glPushMatrix();
		glRotatef(-90, 0.0, 1.0, 0.0);
		glTranslatef(g/2, g/12, -g/2);
		gluSphere(gluObj, g/12, 6, 6);
		gluCylinder(gluObj, g/12, g/12, g, 16, 16);
	glPopMatrix();
}
